"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Utils_1 = require("../utils/Utils");
const fs_1 = require("fs");
module.exports = (vorpal) => {
    const vorpalInstance = vorpal
        .command('convert <fn>')
        .description('Converts Csv to I18n and vice versa.')
        .option('--from <from>')
        .option('--to <to>');
    Utils_1.addBaseCliOptions(vorpalInstance);
    vorpalInstance
        .validate((args) => {
        return isValidFunction(args.fn) && isValidOrigin(args.options.from);
    })
        .action(async (args) => {
        const origin = args.options.from;
        let target = args.options.to;
        target = target ? target.replace(/\/?$/, '/') : target;
        switch (args.fn) {
            case 'i18nToCsv':
                {
                    try {
                        const csv = toCsv(fromI18N(origin));
                        fs_1.writeFileSync(`${target || './'}responses.csv`, csv);
                        console.log('Successfully converted I18n to Csv.');
                    }
                    catch (e) {
                        console.log('Something went wrong while convertingfrom I18N. Check the logs below:\n');
                        console.log(e);
                    }
                }
                break;
            case 'csvToI18n':
                {
                    try {
                        const model = toI18N(fromCsv(origin)) || {};
                        for (const locale in model) {
                            if (!model.hasOwnProperty(locale)) {
                                continue;
                            }
                            fs_1.writeFileSync(`${target || './i18n/'}${locale}.json`, JSON.stringify(model[locale], null, '\t'));
                            console.log('Successfully converted Csv to I18n.');
                        }
                    }
                    catch (e) {
                        console.log('Something went wrong while converting from Csv. Check the logs below:\n');
                        console.log(e);
                    }
                }
                break;
            default: { }
        }
    });
};
function isValidFunction(fn) {
    if (['i18nToCsv', 'csvToI18n'].indexOf(fn) === -1) {
        console.log(`The function ${fn} is not supported. Please use one of the following:`);
        console.log('* i18nToCsv\n* csvToI18n');
        return false;
    }
    return true;
}
function isValidOrigin(origin) {
    if (!origin) {
        console.log('Orgin has to bet set!');
        return false;
    }
    return true;
}
function fromCsv(path) {
    const csv = fs_1.readFileSync(path, 'utf8').split('\n');
    const [localesStr, ...valueStr] = csv;
    const locales = localesStr.split(',');
    const model = {};
    locales.shift();
    for (const valueS of valueStr) {
        if (!valueS) {
            continue;
        }
        const [key, ...vals] = valueS.split(',');
        for (const [i, locale] of locales.entries()) {
            if (!model[locale]) {
                model[locale] = {};
            }
            if (!vals[i]) {
                continue;
            }
            const v = model[locale][key];
            if (!v && v !== '') {
                model[locale][key] = vals[i];
                continue;
            }
            switch (v.constructor) {
                case Array:
                    {
                        if (vals[i] !== '') {
                            v.push(vals[i]);
                        }
                    }
                    break;
                case String:
                    {
                        model[locale][key] = [v];
                        if (vals[i] !== '') {
                            model[locale][key].push(vals[i]);
                        }
                    }
                    break;
                default: { }
            }
        }
    }
    return model;
}
function toCsv(model) {
    if (!model) {
        throw new Error('Something went wrong!');
    }
    const locales = Object.keys(model);
    const obj = {};
    for (const [i, locale] of locales.entries()) {
        const keys = Object.keys(model[locale]);
        for (const key of keys) {
            const value = model[locale][key] || '';
            switch (value.constructor) {
                case String:
                    {
                        if (!obj[key]) {
                            obj[key] = new Array(locales.length).fill('');
                        }
                        obj[key][i] = model[locale][key];
                    }
                    break;
                case Array:
                    {
                        if (!obj[key]) {
                            obj[key] = [];
                        }
                        if (obj[key].length > 0 && obj[key][0].constructor === String) {
                            obj[key] = [obj[key]];
                        }
                        while (obj[key].length < value.length) {
                            obj[key].push(new Array(locales.length).fill(''));
                        }
                        for (const [k, v] of value.entries()) {
                            obj[key][k][i] = v;
                        }
                    }
                    break;
                case Object:
                    {
                        for (const k in value) {
                            if (!value.hasOwnProperty(k)) {
                                continue;
                            }
                            const v = value[k];
                            const newKey = `${key}.${k}`;
                            if (!obj[newKey]) {
                                obj[newKey] = new Array(locales.length).fill('');
                            }
                            obj[newKey][i] = v;
                        }
                    }
                    break;
                default: { }
            }
        }
    }
    let csv = `key,${locales.join(',')}\n`;
    for (const key in obj) {
        if (obj[key][0].constructor === Array) {
            for (const arr of obj[key]) {
                csv += `${key},${arr.join(',')}\n`;
            }
        }
        else {
            csv += `${key},${obj[key].join(',')}\n`;
        }
    }
    return csv;
}
function fromI18N(path) {
    let files = [];
    if (path.indexOf('.json', path.length - 5) !== -1) {
        const pathArr = path.split('/');
        files.push(pathArr.pop());
        path = pathArr.join('/');
    }
    else {
        files = fs_1.readdirSync(path);
    }
    const model = {};
    files.forEach((locale) => {
        const i18nModel = JSON.parse(fs_1.readFileSync(`${path}/${locale}`, 'utf8'));
        for (const prop in i18nModel) {
            if (!i18nModel.hasOwnProperty(prop)) {
                continue;
            }
            let obj = i18nModel[prop];
            if (prop !== 'translation') {
                obj = i18nModel[prop]['translation'];
                model[`${locale.replace('.json', '')}-${prop}`] = obj;
            }
            else {
                model[locale.replace('.json', '')] = obj;
            }
        }
    });
    return model;
}
function toI18N(model) {
    if (!model) {
        throw new Error('Something went wrong!');
    }
    const i18n = {};
    for (const locale in model) {
        if (!model.hasOwnProperty(locale)) {
            continue;
        }
        const obj = {
            translation: model[locale]
        };
        for (const platform of ['AlexaSkill', 'GoogleAction']) {
            const key = `${locale}-${platform}`;
            if (model[key]) {
                obj[platform] = {
                    translation: model[key]
                };
                delete model[key];
            }
        }
        i18n[locale] = obj;
    }
    return i18n;
}
//# sourceMappingURL=convert.js.map