#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const JovoRenderer_1 = require("../utils/JovoRenderer");
const Listr = require("listr");
const Platforms = require("../utils/Platforms");
const DeployTargets = require("../utils/DeployTargets");
const Prompts_1 = require("../utils/Prompts");
const tasks_1 = require("./tasks");
const Utils_1 = require("../utils/Utils");
const Validator_1 = require("../utils/Validator");
const jovo_cli_core_1 = require("jovo-cli-core");
const project = jovo_cli_core_1.getProject();
module.exports = (vorpal) => {
    let DEBUG = false;
    const vorpalInstance = vorpal
        .command('get <platform>')
        .description('Downloads an existing platform project into the platforms folder.')
        .option('-l, --locale <locale>', 'Locale of the language model.\n\t\t\t\t<en-US|de-DE|etc> Default: all locales')
        .option('\n')
        .option('-t, --target <target>', `Type of data that is downloaded. \n\t\t\t\t${DeployTargets.getDeployExampleText()}`)
        .option('--stage <stage>', 'Takes configuration from <stage>')
        .option('-b, --build', 'Runs build after get. Works only with --reverse')
        .option('-r, --reverse', 'Builds Jovo language model from platfrom specific language model')
        .option('--overwrite', 'Forces overwrite of existing project')
        .option('\n');
    Platforms.addCliOptions('get', vorpalInstance);
    Utils_1.addBaseCliOptions(vorpalInstance);
    vorpalInstance
        .validate((args) => {
        if (!Platforms.validateCliOptions('get', args)) {
            return false;
        }
        return Validator_1.isValidLocale(args.options.locale) &&
            Validator_1.isValidDeployTarget(args.options.target);
    })
        .action(async (args) => {
        try {
            DEBUG = args.options.debug ? true : false;
            await project.init();
            const types = [];
            if (args.platform) {
                types.push(args.platform);
            }
            else {
                types.push.apply(types, Platforms.getAll(args.platform, args.options.stage));
            }
            const tasks = new Listr([], {
                renderer: JovoRenderer_1.JovoCliRenderer,
                collapse: false,
            });
            try {
                project.getConfig(args.options.stage);
            }
            catch (e) {
                console.log('\n\n Could not load app.json. \n\n');
                return Promise.resolve();
            }
            const config = {
                types,
                debug: DEBUG,
            };
            for (const type of config.types) {
                const platform = Platforms.get(type);
                let platformConfigIds = platform.getPlatformConfigIds(project, {});
                if (!args.options.overwrite && Object.keys(platformConfigIds).length) {
                    const answers = await Prompts_1.promptOverwriteProjectFiles();
                    if (answers.overwrite === Prompts_1.ANSWER_CANCEL) {
                        return Promise.resolve();
                    }
                }
                platformConfigIds = platform.getPlatformConfigIds(project, args.options);
                _.merge(config, platformConfigIds);
                _.merge(config, platform.getPlatformConfigValues(project, args.options));
                _.merge(config, {
                    locales: project.getLocales(args.options.locale),
                    targets: project.getTargets('get', args.options.target, args.options.stage),
                    stage: project.getStage(args.options.stage),
                });
                if (Object.keys(platformConfigIds).length === 0) {
                    const choices = await platform.getExistingProjects(config);
                    const answers = await Prompts_1.promptListForProjectId(choices);
                    config[platform.constructor.ID_KEY] = answers.id;
                }
                if (args.options.reverse) {
                    try {
                        config.locales = platform.getLocales(args.options.locale);
                    }
                    catch (e) {
                        config.locales = undefined;
                    }
                    if (args.options.overwrite) {
                        config.reverse = true;
                    }
                    else if (project.hasModelFiles(config.locales)) {
                        const answers = await Prompts_1.promptOverwriteReverseBuild();
                        if (answers.promptOverwriteReverseBuild === Prompts_1.ANSWER_CANCEL) {
                            return Promise.resolve();
                        }
                        else {
                            config.reverse = answers.promptOverwriteReverseBuild;
                        }
                    }
                }
                tasks_1.getTask(config).forEach((t) => tasks.add(t));
            }
            if (args.options.build &&
                args.options.reverse) {
                tasks.add({
                    title: 'Building language model platform model',
                    task: (ctx) => tasks_1.buildReverseTask(ctx),
                });
            }
            await tasks.run(config);
            console.log();
            console.log('  Get completed.');
            console.log();
        }
        catch (err) {
            console.error('There was a problem:');
            console.error(err);
            process.exit(1);
        }
    });
};
//# sourceMappingURL=get.js.map